<?php

namespace Crm\Funil\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Crm\Funil\Http\Requests\FunilRequest;
use Crm\Funil\Interfaces\FunilRepositoryInterface;
use Crm\Funil\Models\Funil;

/**
 * Resource controller class for funil.
 */
class FunilResourceController extends BaseController
{

    /**
     * Initialize funil resource controller.
     *
     * @param type FunilRepositoryInterface $funil
     *
     * @return null
     */
    public function __construct(FunilRepositoryInterface $funil)
    {
        parent::__construct();
        $this->repository = $funil;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Crm\Funil\Repositories\Criteria\FunilResourceCriteria::class);
    }

    /**
     * Display a list of funil.
     *
     * @return Response
     */
    public function index(FunilRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Crm\Funil\Repositories\Presenter\FunilPresenter::class)
                ->$function();
        }

        $funils = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('funil::funil.names'))
            ->view('funil::funil.index', true)
            ->data(compact('funils', 'view'))
            ->output();
    }

    /**
     * Display funil.
     *
     * @param Request $request
     * @param Model   $funil
     *
     * @return Response
     */
    public function show(FunilRequest $request, Funil $funil)
    {

        if ($funil->exists) {
            $view = 'funil::funil.show';
        } else {
            $view = 'funil::funil.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('funil::funil.name'))
            ->data(compact('funil'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new funil.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(FunilRequest $request)
    {

        $funil = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('funil::funil.name')) 
            ->view('funil::funil.create', true) 
            ->data(compact('funil'))
            ->output();
    }

    /**
     * Create new funil.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(FunilRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $funil                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('funil::funil.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('funil/funil/' . $funil->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/funil/funil'))
                ->redirect();
        }

    }

    /**
     * Show funil for editing.
     *
     * @param Request $request
     * @param Model   $funil
     *
     * @return Response
     */
    public function edit(FunilRequest $request, Funil $funil)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('funil::funil.name'))
            ->view('funil::funil.edit', true)
            ->data(compact('funil'))
            ->output();
    }

    /**
     * Update the funil.
     *
     * @param Request $request
     * @param Model   $funil
     *
     * @return Response
     */
    public function update(FunilRequest $request, Funil $funil)
    {
        try {
            $attributes = $request->all();

            $funil->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('funil::funil.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('funil/funil/' . $funil->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('funil/funil/' . $funil->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the funil.
     *
     * @param Model   $funil
     *
     * @return Response
     */
    public function destroy(FunilRequest $request, Funil $funil)
    {
        try {

            $funil->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('funil::funil.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('funil/funil/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('funil/funil/' . $funil->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple funil.
     *
     * @param Model   $funil
     *
     * @return Response
     */
    public function delete(FunilRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('funil::funil.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('funil/funil'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/funil/funil'))
                ->redirect();
        }

    }

    /**
     * Restore deleted funils.
     *
     * @param Model   $funil
     *
     * @return Response
     */
    public function restore(FunilRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('funil::funil.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/funil/funil'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/funil/funil/'))
                ->redirect();
        }

    }

}
